// Generated, do not edit by hand.
#include "python/oodyn.hh"

#include <vcsn/dyn/algos.hh>
#include <vcsn/misc/raise.hh>
#include <vcsn/misc/stream.hh>

/// Convert a vector of oodyn values to a vector of dyn values.
template <typename T>
auto make_dyn_vector(const std::vector<T>& v)
{
  auto res = std::vector<decltype(std::declval<T>().val_)>{};
  for (const auto& e: v)
    res.emplace_back(e.val_);
  return res;
}

/// Create an input stream from a file, or from a string.
static
auto make_istream(const std::string& data = "",
                  const std::string& filename = "")
  -> std::shared_ptr<std::istream>
{
  vcsn::require(!data.empty() || !filename.empty(),
                "cannot provide both data and filename");
  if (!data.empty())
    return std::make_shared<std::istringstream>(data);
  else if (!filename.empty())
    return vcsn::open_input_file(filename);
  else
    vcsn::raise("must provide either data or filename");
}


  automaton::automaton(const std::string& data, const std::string& format,
                       const std::string& filename, bool strip)
    : val_{nullptr}
  {
    auto is = make_istream(data, filename);
    try
      {
        val_ = vcsn::dyn::read_automaton(*is, format, strip);
        vcsn::require(is->peek() == EOF, "unexpected trailing characters: ", *is);
      }
    catch (const std::runtime_error& e)
      {
        if (data.empty())
          vcsn::raise(e, "  while reading automaton: ", filename);
        else if (std::count(data.begin(), data.end(), '\n') < 30)
          vcsn::raise(e, "  while reading automaton: ", data);
        else
          vcsn::raise(e, "  while reading automaton");
      }
  }
  automaton automaton::accessible() const
  {
    return vcsn::dyn::accessible(val_);
  }

  automaton automaton::add(const automaton& rhs, const std::string& algo) const
  {
    return vcsn::dyn::add(val_, rhs.val_, algo);
  }

  label automaton::ambiguous_word() const
  {
    return vcsn::dyn::ambiguous_word(val_);
  }

  automaton automaton::coaccessible() const
  {
    return vcsn::dyn::coaccessible(val_);
  }

  automaton automaton::codeterminize(const std::string& algo) const
  {
    return vcsn::dyn::codeterminize(val_, algo);
  }

  automaton automaton::cominimize(const std::string& algo) const
  {
    return vcsn::dyn::cominimize(val_, algo);
  }

  automaton automaton::complement() const
  {
    return vcsn::dyn::complement(val_);
  }

  automaton automaton::complete() const
  {
    return vcsn::dyn::complete(val_);
  }

  automaton automaton::component(unsigned num) const
  {
    return vcsn::dyn::component(val_, num);
  }

  automaton automaton::compose(const automaton& rhs, bool lazy) const
  {
    return vcsn::dyn::compose(val_, rhs.val_, lazy);
  }

  automaton automaton::condense() const
  {
    return vcsn::dyn::condense(val_);
  }

  automaton automaton::conjugate() const
  {
    return vcsn::dyn::conjugate(val_);
  }

  automaton automaton::conjunction(const automaton& rhs, bool lazy) const
  {
    return vcsn::dyn::conjunction(val_, rhs.val_, lazy);
  }

  automaton automaton::conjunction(const std::vector<automaton>& as, bool lazy)
  {
    return vcsn::dyn::conjunction(make_dyn_vector<automaton>(as), lazy);
  }

  automaton automaton::conjunction(unsigned n) const
  {
    return vcsn::dyn::conjunction(val_, n);
  }

  context automaton::context() const
  {
    return vcsn::dyn::context_of(val_);
  }

  automaton automaton::copy() const
  {
    return vcsn::dyn::copy(val_);
  }

  automaton automaton::copy(const struct context& ctx) const
  {
    return vcsn::dyn::copy(val_, ctx.val_);
  }

  automaton automaton::costandard() const
  {
    return vcsn::dyn::costandard(val_);
  }

  automaton automaton::delay_automaton() const
  {
    return vcsn::dyn::delay_automaton(val_);
  }

  automaton automaton::determinize(const std::string& algo) const
  {
    return vcsn::dyn::determinize(val_, algo);
  }

  automaton automaton::difference(const automaton& rhs) const
  {
    return vcsn::dyn::difference(val_, rhs.val_);
  }

  automaton automaton::eliminate_state(int s) const
  {
    return vcsn::dyn::eliminate_state(val_, s);
  }

  weight automaton::eval(const label& l) const
  {
    return vcsn::dyn::eval(val_, l.val_);
  }

  automaton automaton::factor() const
  {
    return vcsn::dyn::factor(val_);
  }

  automaton automaton::filter(const std::vector<unsigned>& ss) const
  {
    return vcsn::dyn::filter(val_, ss);
  }

  automaton automaton::focus(unsigned tape) const
  {
    return vcsn::dyn::focus(val_, tape);
  }

  bool automaton::has_bounded_lag() const
  {
    return vcsn::dyn::has_bounded_lag(val_);
  }

  bool automaton::has_lightening_cycle() const
  {
    return vcsn::dyn::has_lightening_cycle(val_);
  }

  bool automaton::has_twins_property() const
  {
    return vcsn::dyn::has_twins_property(val_);
  }

  automaton automaton::infiltrate(const automaton& rhs) const
  {
    return vcsn::dyn::infiltrate(val_, rhs.val_);
  }

  automaton automaton::infiltrate(const std::vector<automaton>& as)
  {
    return vcsn::dyn::infiltrate(make_dyn_vector<automaton>(as));
  }

  automaton automaton::insplit(bool lazy) const
  {
    return vcsn::dyn::insplit(val_, lazy);
  }

  bool automaton::is_accessible() const
  {
    return vcsn::dyn::is_accessible(val_);
  }

  bool automaton::is_ambiguous() const
  {
    return vcsn::dyn::is_ambiguous(val_);
  }

  bool automaton::is_coaccessible() const
  {
    return vcsn::dyn::is_coaccessible(val_);
  }

  bool automaton::is_codeterministic() const
  {
    return vcsn::dyn::is_codeterministic(val_);
  }

  bool automaton::is_complete() const
  {
    return vcsn::dyn::is_complete(val_);
  }

  bool automaton::is_costandard() const
  {
    return vcsn::dyn::is_costandard(val_);
  }

  bool automaton::is_cycle_ambiguous() const
  {
    return vcsn::dyn::is_cycle_ambiguous(val_);
  }

  bool automaton::is_deterministic() const
  {
    return vcsn::dyn::is_deterministic(val_);
  }

  bool automaton::is_empty() const
  {
    return vcsn::dyn::is_empty(val_);
  }

  bool automaton::is_eps_acyclic() const
  {
    return vcsn::dyn::is_eps_acyclic(val_);
  }

  bool automaton::is_equivalent(const automaton& rhs) const
  {
    return vcsn::dyn::are_equivalent(val_, rhs.val_);
  }

  bool automaton::is_functional() const
  {
    return vcsn::dyn::is_functional(val_);
  }

  bool automaton::is_isomorphic(const automaton& rhs) const
  {
    return vcsn::dyn::are_isomorphic(val_, rhs.val_);
  }

  bool automaton::is_letterized() const
  {
    return vcsn::dyn::is_letterized(val_);
  }

  bool automaton::is_normalized() const
  {
    return vcsn::dyn::is_normalized(val_);
  }

  bool automaton::is_out_sorted() const
  {
    return vcsn::dyn::is_out_sorted(val_);
  }

  bool automaton::is_partial_identity() const
  {
    return vcsn::dyn::is_partial_identity(val_);
  }

  bool automaton::is_proper() const
  {
    return vcsn::dyn::is_proper(val_);
  }

  bool automaton::is_realtime() const
  {
    return vcsn::dyn::is_realtime(val_);
  }

  bool automaton::is_standard() const
  {
    return vcsn::dyn::is_standard(val_);
  }

  bool automaton::is_synchronized() const
  {
    return vcsn::dyn::is_synchronized(val_);
  }

  bool automaton::is_synchronized_by(const label& word) const
  {
    return vcsn::dyn::is_synchronized_by(val_, word.val_);
  }

  bool automaton::is_synchronizing() const
  {
    return vcsn::dyn::is_synchronizing(val_);
  }

  bool automaton::is_trim() const
  {
    return vcsn::dyn::is_trim(val_);
  }

  bool automaton::is_useless() const
  {
    return vcsn::dyn::is_useless(val_);
  }

  bool automaton::is_valid() const
  {
    return vcsn::dyn::is_valid(val_);
  }

  automaton automaton::ldivide(const automaton& rhs) const
  {
    return vcsn::dyn::ldivide(val_, rhs.val_);
  }

  automaton automaton::letterize() const
  {
    return vcsn::dyn::letterize(val_);
  }

  automaton automaton::lift(const std::vector<unsigned>& tapes, identities ids) const
  {
    return vcsn::dyn::lift(val_, tapes, ids);
  }

  polynomial automaton::lightest(unsigned num, const std::string& algo) const
  {
    return vcsn::dyn::lightest(val_, num, algo);
  }

  automaton automaton::lightest_automaton(unsigned num, const std::string& algo) const
  {
    return vcsn::dyn::lightest_automaton(val_, num, algo);
  }

  automaton automaton::lweight(const weight& w, const std::string& algo) const
  {
    return vcsn::dyn::lweight(w.val_, val_, algo);
  }

  automaton automaton::minimize(const std::string& algo) const
  {
    return vcsn::dyn::minimize(val_, algo);
  }

  automaton automaton::multiply(const automaton& rhs, const std::string& algo) const
  {
    return vcsn::dyn::multiply(val_, rhs.val_, algo);
  }

  automaton automaton::multiply(int min, int max, const std::string& algo) const
  {
    return vcsn::dyn::multiply(val_, min, max, algo);
  }

  automaton automaton::normalize() const
  {
    return vcsn::dyn::normalize(val_);
  }

  std::size_t automaton::num_components() const
  {
    return vcsn::dyn::num_components(val_);
  }

  automaton automaton::pair(bool keep_initials) const
  {
    return vcsn::dyn::pair(val_, keep_initials);
  }

  automaton automaton::partial_identity() const
  {
    return vcsn::dyn::partial_identity(val_);
  }

  automaton automaton::prefix() const
  {
    return vcsn::dyn::prefix(val_);
  }

  automaton automaton::project(unsigned tape) const
  {
    return vcsn::dyn::project(val_, tape);
  }

  automaton automaton::proper(direction dir, bool prune, const std::string& algo) const
  {
    return vcsn::dyn::proper(val_, dir, prune, algo);
  }

  automaton automaton::push_weights() const
  {
    return vcsn::dyn::push_weights(val_);
  }

  automaton automaton::rdivide(const automaton& rhs) const
  {
    return vcsn::dyn::rdivide(val_, rhs.val_);
  }

  automaton automaton::read_automaton(std::istream& is, const std::string& format, bool strip)
  {
    return vcsn::dyn::read_automaton(is, format, strip);
  }

  automaton automaton::realtime() const
  {
    return vcsn::dyn::realtime(val_);
  }

  automaton automaton::reduce() const
  {
    return vcsn::dyn::reduce(val_);
  }

  automaton automaton::rweight(const weight& w, const std::string& algo) const
  {
    return vcsn::dyn::rweight(val_, w.val_, algo);
  }

  automaton automaton::scc(const std::string& algo) const
  {
    return vcsn::dyn::scc(val_, algo);
  }

  polynomial automaton::shortest(boost::optional<unsigned> num, boost::optional<unsigned> len) const
  {
    return vcsn::dyn::shortest(val_, num, len);
  }

  automaton automaton::shuffle(const automaton& rhs) const
  {
    return vcsn::dyn::shuffle(val_, rhs.val_);
  }

  automaton automaton::shuffle(const std::vector<automaton>& as)
  {
    return vcsn::dyn::shuffle(make_dyn_vector<automaton>(as));
  }

  automaton automaton::sort() const
  {
    return vcsn::dyn::sort(val_);
  }

  automaton automaton::standard() const
  {
    return vcsn::dyn::standard(val_);
  }

  automaton automaton::star(const std::string& algo) const
  {
    return vcsn::dyn::star(val_, algo);
  }

  automaton automaton::strip() const
  {
    return vcsn::dyn::strip(val_);
  }

  automaton automaton::subword() const
  {
    return vcsn::dyn::subword(val_);
  }

  automaton automaton::suffix() const
  {
    return vcsn::dyn::suffix(val_);
  }

  automaton automaton::synchronize() const
  {
    return vcsn::dyn::synchronize(val_);
  }

  label automaton::synchronizing_word(const std::string& algo) const
  {
    return vcsn::dyn::synchronizing_word(val_, algo);
  }

  expression automaton::to_expression(identities ids, const std::string& algo) const
  {
    return vcsn::dyn::to_expression(val_, ids, algo);
  }

  automaton automaton::transpose()
  {
    return vcsn::dyn::transpose(val_);
  }

  automaton automaton::trim() const
  {
    return vcsn::dyn::trim(val_);
  }

  automaton automaton::tuple(const automaton& rhs) const
  {
    return vcsn::dyn::tuple(val_, rhs.val_);
  }

  automaton automaton::tuple(const std::vector<automaton>& as)
  {
    return vcsn::dyn::tuple(make_dyn_vector<automaton>(as));
  }

  std::string automaton::type() const
  {
    return vcsn::dyn::type(val_);
  }

  automaton automaton::universal() const
  {
    return vcsn::dyn::universal(val_);
  }

  weight automaton::weight_series() const
  {
    return vcsn::dyn::weight_series(val_);
  }


  context::context()
  {}

  context::context(const std::string& ctx)
    : context(vcsn::dyn::make_context(ctx))
  {}

  context::operator bool() const
  {
    return bool(val_);
  }

  automaton context::cotrie(const std::string& data,
                            const std::string& format,
                            const std::string& filename) const
  {
    auto is = make_istream(data, filename);
    auto res = cotrie(*is, format);
    vcsn::require(is->peek() == EOF, "unexpected trailing characters: ", *is);
    return res;
  }

  automaton context::trie(const std::string& data,
                          const std::string& format,
                          const std::string& filename) const
  {
    auto is = make_istream(data, filename);
    auto res = trie(*is, format);
    vcsn::require(is->peek() == EOF, "unexpected trailing characters: ", *is);
    return res;
  }
  automaton context::cerny(unsigned num_states) const
  {
    return vcsn::dyn::cerny(val_, num_states);
  }

  automaton context::cotrie(std::istream& is, const std::string& format) const
  {
    return vcsn::dyn::cotrie(val_, is, format);
  }

  automaton context::de_bruijn(unsigned n) const
  {
    return vcsn::dyn::de_bruijn(val_, n);
  }

  automaton context::divkbaseb(unsigned divisor, unsigned base) const
  {
    return vcsn::dyn::divkbaseb(val_, divisor, base);
  }

  automaton context::double_ring(unsigned n, const std::vector<unsigned>& f) const
  {
    return vcsn::dyn::double_ring(val_, n, f);
  }

  expression context::expression_one(identities ids) const
  {
    return vcsn::dyn::expression_one(val_, ids);
  }

  expression context::expression_zero(identities ids) const
  {
    return vcsn::dyn::expression_zero(val_, ids);
  }

  context context::join(const struct context& rhs) const
  {
    return vcsn::dyn::join(val_, rhs.val_);
  }

  automaton context::ladybird(unsigned n) const
  {
    return vcsn::dyn::ladybird(val_, n);
  }

  automaton context::levenshtein() const
  {
    return vcsn::dyn::levenshtein(val_);
  }

  context context::make_context(const std::string& name)
  {
    return vcsn::dyn::make_context(name);
  }

  context context::make_word_context() const
  {
    return vcsn::dyn::make_word_context(val_);
  }

  std::size_t context::num_tapes() const
  {
    return vcsn::dyn::num_tapes(val_);
  }

  context context::project(unsigned tape) const
  {
    return vcsn::dyn::project(val_, tape);
  }

  automaton context::quotkbaseb(unsigned divisor, unsigned base) const
  {
    return vcsn::dyn::quotkbaseb(val_, divisor, base);
  }

  automaton context::random_automaton(unsigned num_states, float density, unsigned num_initial, unsigned num_final, boost::optional<unsigned> max_labels, float loop_chance) const
  {
    return vcsn::dyn::random_automaton(val_, num_states, density, num_initial, num_final, max_labels, loop_chance);
  }

  automaton context::random_automaton_deterministic(unsigned num_states) const
  {
    return vcsn::dyn::random_automaton_deterministic(val_, num_states);
  }

  expression context::random_expression(const std::string& param, identities ids) const
  {
    return vcsn::dyn::random_expression(val_, param, ids);
  }

  expression context::read_expression(identities ids, std::istream& is, const std::string& format) const
  {
    return vcsn::dyn::read_expression(val_, ids, is, format);
  }

  label context::read_label(std::istream& is, const std::string& format) const
  {
    return vcsn::dyn::read_label(val_, is, format);
  }

  polynomial context::read_polynomial(std::istream& is) const
  {
    return vcsn::dyn::read_polynomial(val_, is);
  }

  weight context::read_weight(std::istream& is) const
  {
    return vcsn::dyn::read_weight(val_, is);
  }

  expression context::to_expression(identities ids, const label& l) const
  {
    return vcsn::dyn::to_expression(val_, ids, l.val_);
  }

  expression context::to_expression(identities ids, const vcsn::letter_class_t& letters, bool accept) const
  {
    return vcsn::dyn::to_expression(val_, ids, letters, accept);
  }

  automaton context::trie(std::istream& is, const std::string& format) const
  {
    return vcsn::dyn::trie(val_, is, format);
  }

  automaton context::u(unsigned n) const
  {
    return vcsn::dyn::u(val_, n);
  }

  expansion expansion::add(const expansion& rhs) const
  {
    return vcsn::dyn::add(val_, rhs.val_);
  }

  expansion expansion::complement() const
  {
    return vcsn::dyn::complement(val_);
  }

  expansion expansion::compose(const expansion& rhs) const
  {
    return vcsn::dyn::compose(val_, rhs.val_);
  }

  expansion expansion::conjunction(const expansion& rhs) const
  {
    return vcsn::dyn::conjunction(val_, rhs.val_);
  }

  context expansion::context() const
  {
    return vcsn::dyn::context_of(val_);
  }

  expansion expansion::lweight(const weight& w) const
  {
    return vcsn::dyn::lweight(w.val_, val_);
  }

  expansion expansion::project(unsigned tape) const
  {
    return vcsn::dyn::project(val_, tape);
  }

  expansion expansion::rweight(const weight& w) const
  {
    return vcsn::dyn::rweight(val_, w.val_);
  }

  expansion expansion::tuple(const std::vector<expansion>& rs)
  {
    return vcsn::dyn::tuple(make_dyn_vector<expansion>(rs));
  }


  expression::expression(const struct context& ctx, const std::string& r,
                         identities ids)
  {
    std::istringstream is{r};
    try
      {
        val_ = vcsn::dyn::read_expression(ctx.val_, ids, is);
        vcsn::require(is.peek() == EOF, "unexpected trailing characters: ", is);
      }
    catch (const std::runtime_error& e)
      {
        vcsn::raise(e, "  while reading expression: ", r);
      }
  }

  expression expression::as(const struct context& ctx,
                            identities ids) const
  {
    return copy((ctx ? ctx : context()), ids);
  }

  expression expression::add(const expression& rhs) const
  {
    return vcsn::dyn::add(val_, rhs.val_);
  }

  expression expression::complement() const
  {
    return vcsn::dyn::complement(val_);
  }

  expression expression::compose(const expression& rhs) const
  {
    return vcsn::dyn::compose(val_, rhs.val_);
  }

  expression expression::concatenate(const expression& rhs) const
  {
    return vcsn::dyn::concatenate(val_, rhs.val_);
  }

  expression expression::conjunction(const expression& rhs) const
  {
    return vcsn::dyn::conjunction(val_, rhs.val_);
  }

  weight expression::constant_term() const
  {
    return vcsn::dyn::constant_term(val_);
  }

  context expression::context() const
  {
    return vcsn::dyn::context_of(val_);
  }

  expression expression::copy(const struct context& ctx, identities ids) const
  {
    return vcsn::dyn::copy(val_, ctx.val_, ids);
  }

  polynomial expression::derivation(const label& lbl, bool breaking) const
  {
    return vcsn::dyn::derivation(val_, lbl.val_, breaking);
  }

  automaton expression::derived_term(const std::string& algo) const
  {
    return vcsn::dyn::derived_term(val_, algo);
  }

  expression expression::difference(const expression& rhs) const
  {
    return vcsn::dyn::difference(val_, rhs.val_);
  }

  expression expression::expand() const
  {
    return vcsn::dyn::expand(val_);
  }

  identities expression::identities_of() const
  {
    return vcsn::dyn::identities_of(val_);
  }

  automaton expression::inductive(const std::string& algo) const
  {
    return vcsn::dyn::inductive(val_, algo);
  }

  expression expression::infiltrate(const expression& rhs) const
  {
    return vcsn::dyn::infiltrate(val_, rhs.val_);
  }

  bool expression::is_equivalent(const expression& rhs) const
  {
    return vcsn::dyn::are_equivalent(val_, rhs.val_);
  }

  bool expression::is_valid() const
  {
    return vcsn::dyn::is_valid(val_);
  }

  expression expression::ldivide(const expression& rhs) const
  {
    return vcsn::dyn::ldivide(val_, rhs.val_);
  }

  bool expression::less_than(const expression& rhs) const
  {
    return vcsn::dyn::less_than(val_, rhs.val_);
  }

  expression expression::lift() const
  {
    return vcsn::dyn::lift(val_);
  }

  expression expression::lweight(const weight& w) const
  {
    return vcsn::dyn::lweight(w.val_, val_);
  }

  expression expression::multiply(const expression& rhs) const
  {
    return vcsn::dyn::multiply(val_, rhs.val_);
  }

  expression expression::multiply(int min, int max) const
  {
    return vcsn::dyn::multiply(val_, min, max);
  }

  expression expression::project(unsigned tape) const
  {
    return vcsn::dyn::project(val_, tape);
  }

  expression expression::rdivide(const expression& rhs) const
  {
    return vcsn::dyn::rdivide(val_, rhs.val_);
  }

  expression expression::rweight(const weight& w) const
  {
    return vcsn::dyn::rweight(val_, w.val_);
  }

  expression expression::shuffle(const expression& rhs) const
  {
    return vcsn::dyn::shuffle(val_, rhs.val_);
  }

  polynomial expression::split() const
  {
    return vcsn::dyn::split(val_);
  }

  automaton expression::standard() const
  {
    return vcsn::dyn::standard(val_);
  }

  unsigned expression::star_height() const
  {
    return vcsn::dyn::star_height(val_);
  }

  expression expression::star_normal_form() const
  {
    return vcsn::dyn::star_normal_form(val_);
  }

  automaton expression::thompson() const
  {
    return vcsn::dyn::thompson(val_);
  }

  automaton expression::to_automaton(const std::string& algo) const
  {
    return vcsn::dyn::to_automaton(val_, algo);
  }

  expansion expression::to_expansion() const
  {
    return vcsn::dyn::to_expansion(val_);
  }

  expression expression::transpose() const
  {
    return vcsn::dyn::transpose(val_);
  }

  expression expression::transposition() const
  {
    return vcsn::dyn::transposition(val_);
  }

  expression expression::tuple(const expression& rhs) const
  {
    return vcsn::dyn::tuple(val_, rhs.val_);
  }

  expression expression::tuple(const std::vector<expression>& rs)
  {
    return vcsn::dyn::tuple(make_dyn_vector<expression>(rs));
  }

  automaton expression::zpc(const std::string& algo) const
  {
    return vcsn::dyn::zpc(val_, algo);
  }



  label::label(const context& ctx, const std::string& s)
  {
    std::istringstream is{s};
    try
      {
        val_ = vcsn::dyn::read_label(ctx.val_, is);
        vcsn::require(is.peek() == EOF, "unexpected trailing characters: ", is);
      }
    catch (const std::runtime_error& e)
      {
        vcsn::raise(e, "  while reading label: ", s);
      }
  }
  label label::multiply(const label& rhs) const
  {
    return vcsn::dyn::multiply(val_, rhs.val_);
  }

  label label::multiply(int num) const
  {
    return vcsn::dyn::multiply(val_, num);
  }

  label label::project(unsigned tape) const
  {
    return vcsn::dyn::project(val_, tape);
  }


  polynomial::polynomial(const struct context& ctx, const std::string& s)
  {
    std::istringstream is{s};
    try
      {
        val_ = vcsn::dyn::read_polynomial(ctx.val_, is);
        vcsn::require(is.peek() == EOF, "unexpected trailing characters: ", is);
      }
    catch (const std::runtime_error& e)
      {
        vcsn::raise(e, "  while reading polynomial: ", s);
      }
  }
  polynomial polynomial::add(const polynomial& rhs) const
  {
    return vcsn::dyn::add(val_, rhs.val_);
  }

  polynomial polynomial::compose(const polynomial& rhs) const
  {
    return vcsn::dyn::compose(val_, rhs.val_);
  }

  polynomial polynomial::conjunction(const polynomial& rhs) const
  {
    return vcsn::dyn::conjunction(val_, rhs.val_);
  }

  context polynomial::context() const
  {
    return vcsn::dyn::context_of(val_);
  }

  automaton polynomial::cotrie() const
  {
    return vcsn::dyn::cotrie(val_);
  }

  polynomial polynomial::ldivide(const polynomial& rhs) const
  {
    return vcsn::dyn::ldivide(val_, rhs.val_);
  }

  polynomial polynomial::lgcd(const polynomial& rhs) const
  {
    return vcsn::dyn::lgcd(val_, rhs.val_);
  }

  polynomial polynomial::lweight(const weight& w) const
  {
    return vcsn::dyn::lweight(w.val_, val_);
  }

  polynomial polynomial::multiply(const polynomial& rhs) const
  {
    return vcsn::dyn::multiply(val_, rhs.val_);
  }

  polynomial polynomial::project(unsigned tape) const
  {
    return vcsn::dyn::project(val_, tape);
  }

  polynomial polynomial::rweight(const weight& w) const
  {
    return vcsn::dyn::rweight(val_, w.val_);
  }

  polynomial polynomial::split() const
  {
    return vcsn::dyn::split(val_);
  }

  automaton polynomial::trie() const
  {
    return vcsn::dyn::trie(val_);
  }

  polynomial polynomial::tuple(const std::vector<polynomial>& ps)
  {
    return vcsn::dyn::tuple(make_dyn_vector<polynomial>(ps));
  }


  weight::weight(const context& ctx, const std::string& s)
  {
    std::istringstream is(s);
    try
      {
        val_ = vcsn::dyn::read_weight(ctx.val_, is);
        vcsn::require(is.peek() == EOF, "unexpected trailing characters: ", is);
      }
    catch (const std::runtime_error& e)
      {
        vcsn::raise(e, "  while reading weight: ", s);
      }
  }
  weight weight::add(const weight& rhs) const
  {
    return vcsn::dyn::add(val_, rhs.val_);
  }

  weight weight::multiply(const weight& rhs) const
  {
    return vcsn::dyn::multiply(val_, rhs.val_);
  }

  weight weight::multiply(int min, int max) const
  {
    return vcsn::dyn::multiply(val_, min, max);
  }


